using System;
using System.Collections;
using System.Text;

namespace RepPP.Toolkit {

    //******************************************************************
    //
    /// <summary>
    /// Synchronize the access to a transaction 
    /// </summary>
    /// <remarks>
    /// This class ensure no data has been modified by another user since the data
    /// has been read. To do it, this class use a field mark for auto date stamping
    /// at update. When the data is read, the datetime is kept. Before the card is
    /// update, the class READ FOR UPDATE the datetime and compare it with the kept
    /// value. If value is different, the DataChanged method is called which in the
    /// base implementation trigger an exception.
    /// </remarks>
    //
    //******************************************************************
    public class DataChangedDetector : IDisposable {
        /// <summary>Sent when the data change by another user since it has been read</summary>
        public event RepPP.DbBeforeUpdCardEventHandler  DataChanged;
        /// <summary>Connection use to access the database</summary>
        private RepPP.Connection                        m_conn;
        /// <summary>Card instance monitored</summary>
        private RepPP.CardInstance                      m_cinst;
        /// <summary>Card group in the card instance being monitored</summary>
        private RepPP.CardGroup                         m_cgrp;
        /// <summary>SQL command use to read for update the last modification date</summary>
        private string                                  m_strReadForUpdate;
        /// <summary>Field containing the date stamping</summary>
        private RepPP.Field                             m_fldLastUpdate;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="cinst">                Card instance to monitor</param>
        /// <param name="cgrp">                 Group in the card instance to monitor. Null for root.</param>
        /// <param name="conn">                 Connection to use to access the data</param>
        //
        //******************************************************************
        public DataChangedDetector(RepPP.CardInstance cinst, RepPP.CardGroup cgrp, RepPP.Connection conn) {
            m_conn          = (conn == null) ? cgrp.Application.DataConnection : conn;
            m_cgrp          = cgrp;
            if (m_cgrp == null) {
                throw new System.ApplicationException("The card " + cinst.Card.Name + " must have a root card group");
            }
            CardInstance    = cinst;
            m_fldLastUpdate = null;
            foreach (RepPP.Field fld in m_cgrp.Group.Fields) {
                if ((fld.Options & RepPP.FieldOptions.sdFieldAutoUpdTimeStamp) != 0) {
                    m_fldLastUpdate = fld;
                    break;
                }
            }
            if (m_fldLastUpdate == null) {
                throw new System.ApplicationException("The group " + m_cgrp.Group.Name + " must contains a field with automatic date stamping at update");
            }
            m_strReadForUpdate  = "SELECT " + m_fldLastUpdate.ColPhysName + " FROM " + m_cgrp.Group.TableName + 
                                  " " + Util.BuildPKWhereClause(m_cgrp, true) + 
                                  " FOR UPDATE";
            m_cgrp.Application.DbBeforeUpdCard += new DbBeforeUpdCardEventHandler(Application_DbBeforeUpdCard);
        }

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="cgrp">                 Group in the card instance to monitor. Null for root.</param>
        /// <param name="conn">                 Connection to use to access the data</param>
        //
        //******************************************************************
        public DataChangedDetector(RepPP.CardGroup cgrp, RepPP.Connection conn) : this(null, cgrp, conn) {
        }

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                            Dispose the object
        /// </summary>
        //
        //******************************************************************
        public void Dispose() {
            if (m_cgrp != null) {
                m_cgrp.Application.DbBeforeUpdCard -= new DbBeforeUpdCardEventHandler(Application_DbBeforeUpdCard);
                m_cgrp = null;
            }
            m_cinst = null;
        }

        //******************************************************************
        //
        /// <summary>
        /// CardInstance:                       Card instance monitored by this class
        /// </summary>
        //
        //******************************************************************
        public RepPP.CardInstance CardInstance {
            get {
                return(m_cinst);
            }
            set {
                if (value != null && value.Card != m_cgrp.Card) {
                    throw new System.ApplicationException("The card group " + m_cgrp.Group.Name + " is not attached to the card " + m_cinst.Card.Name);
                }
                m_cinst = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CardGroup:                          Card group in the card monitored by this class
        /// </summary>
        //
        //******************************************************************
        public RepPP.CardGroup CardGroup {
            get {
                return(m_cgrp);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ReadForUpdateSqlCmd:                SQL command used to READ for update the last modification date
        /// </summary>
        //
        //******************************************************************
        public string ReadForUpdateSqlCmd {
            get {
                return(m_strReadForUpdate);
            }
            set {
                m_strReadForUpdate = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnDataChanged:                      Called when data has been changed since it has been read
        /// </summary>
        /// <param name="e">                    Event arguments</param>
        //
        //******************************************************************
        protected virtual void OnDataChanged(RepPP.CardEventArgs e) {
            if (DataChanged != null) {
                DataChanged(this, e);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CheckActiveLine:                    Checks for the active line if the datetime has changed
        /// </summary>
        /// <param name="ginst">                Group instance to check</param>
        /// <returns>
        /// true if succeed, false if date time has changed
        /// </returns>
        //
        //******************************************************************
        private bool CheckActiveLine(RepPP.GroupInstance ginst) {
            bool                bRetVal = true;
            RepPP.SqlCommand    sqlCmd;
            RepPP.SqlCursor     sqlCur = null;
            RepPP.FieldInstance fi;
            string              strDateTime;
            DateTime            dateTimeNew;
            DateTime            dateTimeOld;

            fi = ginst.FieldInstances[m_fldLastUpdate.Name];
            if (fi.IsNull()) {
                throw new System.ApplicationException("The automatic date stamping field is null");
            }
            dateTimeOld = fi.GetValueAsDateTime(ginst.CurrentLine);
            sqlCmd      = m_conn.SqlCommands.Open(m_strReadForUpdate, RepPP.SqlOptimization.sdOptCursor);
            if (sqlCmd == null) {
                throw new System.ApplicationException(m_conn.ErrorMessage);
            }
            using (sqlCmd) {
                if (sqlCmd.Execute(ref sqlCur) != 0) {
                    throw new System.ApplicationException(m_conn.ErrorMessage);
                }
                using(sqlCur) {
                    if (sqlCur.Next() != 0) {
                        throw new System.ApplicationException("The record doesn't exist anymore");
                    } else {
                        strDateTime = sqlCur.GetColumnValue(0);
                        if (strDateTime == String.Empty) {
                            throw new System.ApplicationException("The automatic date stamping field is null");
                        }
                        dateTimeNew = RepPP.Field.ConvertValueToDateTime(strDateTime, m_fldLastUpdate.Type, m_fldLastUpdate.MaxSize);
                        if (dateTimeOld != dateTimeNew) {
                            bRetVal = false;
                        }
                    }
                }
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// RecursiveCheck:                     Checks recursively for data changed by another user
        /// </summary>
        /// <param name="ginst">                Group instance to check</param>
        /// <returns>
        /// true if succeed, false if date time has changed
        /// </returns>
        //
        //******************************************************************
        private bool RecursiveCheck(RepPP.GroupInstance ginst) {
            bool                bRetVal = true;
            int                 iLineIndex;
            int                 iLineCount;
            int                 iChildIndex;
            int                 iChildCount;
            RepPP.GroupInstance ginstChild;
            
            iLineCount = ginst.LineCount;
            if (ginst.CardGroup == m_cgrp) {
                for (iLineIndex = 0; iLineIndex < iLineCount; iLineIndex++) {
                    if (ginst.ThisLineState(iLineIndex) == LineState.sdLineChanged) {
                        ginst.SelectLine(iLineIndex, false, false);
                        bRetVal = CheckActiveLine(ginst);
                    }
                }
            } else {
                iLineIndex = 0;
                while (iLineIndex < iLineCount && bRetVal) {
                    if (ginst.ThisLineState(iLineIndex) == LineState.sdLineOld || ginst.ThisLineState(iLineIndex) == LineState.sdLineChanged) {
                        ginst.SelectLine(iLineIndex, false, false);
                        iChildCount = ginst.ChildGroupInstances.Count;
                        iChildIndex = 0;
                        while (iChildIndex < iChildCount && bRetVal) {
                            ginstChild = ginst.GetChild(iChildIndex, false, false);
                            if (ginstChild != null) {
                                ginstChild.Select(false, false);
                                bRetVal = RecursiveCheck(ginstChild);
                            }
                            iChildIndex++;
                        }
                    }
                    iLineIndex++;
                }
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// Application_DbBeforeUpdCard:        Called when data is being update in a card
        /// </summary>
        /// <param name="sender">               Sender object</param>
        /// <param name="e">                    Evenet argument</param>
        //
        //******************************************************************
        void Application_DbBeforeUpdCard(object sender, CardEventArgs e) {
            RepPP.GroupInstance ginst;
            
            if (e.CardInstance == m_cinst) {
                ginst = m_cinst.RootGroupInstance;
                ginst.SaveLinePos();
                try {
                    if (!RecursiveCheck(m_cinst.RootGroupInstance)) {
                        e.EventState = RepPP.EventState.sdReject;
                        m_conn.Rollback();
                        OnDataChanged(e);
                    }
                } finally {
                    ginst.RestoreLinePos(true);
                }
            }
        }
    } // Class DataChangedDetector
} // Namespace
