using System;
using System.Drawing.Design;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
    /// <summary>
    /// Implement the base functionnality of a Rep++ control which inherit from a WEB control and
    /// supports group property.
    /// Because .NET doesn't support multiple inheritance, this class must be embedded
    /// </summary>
    //
    //********************************************************************************
    public class GrpControlUtil : ControlUtil {
	    /// <summary>Common interface for controls attached to a group</summary>
	    public interface IGrpControl : IRepControl {
	        /// <summary>Rep++ application object</summary>
	        RepPP.Application           Application { get; }
	        /// <summary>Field name</summary>
	        string                      GroupName { get; set; }
	        /// <summary>Trigger an event to indicate a component property has changed</summary>
	        void                        TriggerComponentChange();
	    }
	    
	    /// <summary>Field attached to the control</summary>
	    protected Group                 m_grp;
	    /// <summary>Group instance attached to the control</summary>
	    protected GroupInstance         m_ginst;
	    /// <summary>Full name of the field</summary>
	    protected string                m_strGrpName;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        /// <param name="bNoRefCount">      true to disable reference counting for this instance</param>
        //
        //********************************************************************************
        public GrpControlUtil(object parent, bool bResDispenserOnly, bool bNoRefCount) : base(parent, bResDispenserOnly, bNoRefCount) {
		    m_grp        = null;
		    m_ginst      = null;
		    m_strGrpName = "";
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        //
        //********************************************************************************
        public GrpControlUtil(object parent, bool bResDispenserOnly) : this(parent, bResDispenserOnly, false) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// GroupName:              Group name of the field attached to the control
        /// </summary>
        //
        //********************************************************************************
		public string GroupName {
		    get {
		        return(m_strGrpName);
		    }
		    set {
		        m_strGrpName = (value == null) ? "" : value;
		        m_grp        = null;
		        m_ginst      = null;
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// Group:                  Rep++ Group attached to the control
		/// </summary>
        //
        //********************************************************************************
		public Group Group {
		    get {
                Group               grpRetVal = null;
		        RepPP.Application   app;
		        
		        if (m_grp != null && m_grp.IsDetached) {
		            m_grp   = null;
		            m_ginst = null;
		        }
		        if (m_grp != null) {
                    grpRetVal = m_grp;
                } else {
		            app         = Application;
	                grpRetVal   = (app == null || m_strGrpName == String.Empty) ? null : app.Groups[m_strGrpName];
                    if (!IsDesignMode) {
                        m_grp = grpRetVal;
                    }
		        }
		        return(grpRetVal);
		    }
		    set {
		        if (value != m_grp) {
		            m_grp   = null;
		            m_ginst = null;
		            if (value == null) {
		                m_strGrpName = "";
		            } else {
		                m_strGrpName  = value.Name;
		                m_grp         = value;
		            }
		        }
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// GroupInstance:              Rep++ Group instance attached to the control
		/// </summary>
        //
        //********************************************************************************
		public GroupInstance GroupInstance {
		    get {
		        if (m_ginst != null && m_ginst.IsDetached) {
		            m_grp   = null;
		            m_ginst = null;
		        }
		        return(m_ginst);
		    }
		    set {
		        if (value != m_ginst) {
		            m_grp   = null;
		            m_ginst = null;
		            if (value == null) {
		                m_strGrpName = "";
		            } else {
		                m_strGrpName  = value.Group.Name;
		                m_grp         = value.Group;
		                m_ginst       = value;
		            }
		        }
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the Group name property
	    /// </summary>
        //
        //********************************************************************************
        public class GroupNameEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                IGrpControl                     grpControl;
                IWindowsFormsEditorService      edSvc;
                RepPP.Application               app;
                FldControlUtil.PopupListBox     listBox;
                string                          strGrpName;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    app   = RepPP.Application.GetApplicationForDesign(provider);
                    if (edSvc != null && app != null) {
                        grpControl              = context.Instance as IGrpControl;
                        objRetVal               = grpControl.GroupName;
                        listBox                 = new FldControlUtil.PopupListBox(edSvc);
                        listBox.Location        = new System.Drawing.Point(0, 0);
                        listBox.Name            = "listBox";
                        listBox.Size            = new System.Drawing.Size(168, 186);
                        listBox.TabIndex        = 1;
                        listBox.BorderStyle     = System.Windows.Forms.BorderStyle.None;
                        listBox.Sorted          = true;
                        foreach (Group grp in app.Groups) {
                            if (String.Compare(grp.Name, 0, "$HIDE", 0, 5) != 0) {
                                listBox.Items.Add(grp.Name);
                            }
                        }
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            strGrpName = listBox.SelectedItem as string;
                            objRetVal  = strGrpName;
                        }   
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class GroupNameEditor
    } // Class FldControlUtil}
} // Namespace